#!/usr/bin/python
# -*- coding: UTF-8 -*- 

#############################################################################
#  Copyright & License:
#  ====================
#   
#  Copyright 2009-2016 gAGE/UPC & ESA 
#   
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#   
#  http://www.apache.org/licenses/LICENSE-2.0
#   
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#############################################################################

#############################################################################
# Copyright: gAGE/UPC & ESA
# Project: EDUNAV GNSS Lab Tool
# Authors: Adria Rovira-Garcia (group of Astronomy and GEomatics - gAGE/UPC)
#          Pere Ramos-Bosch (group of Astronomy and GEomatics - gAGE/UPC)
# Curator: Adria Rovira-Garcia ( gAGE/UPC )
# Developer: Yixie Shao ( gAGE/UPC )
#            Deimos Ibanez Segura ( gAGE/UPC )
#          glab.gage @ upc.edu           
# File: graph.py
# Code Management Tool File Version: 3.1  Revision: 1
# Date: 2016/10/21
#############################################################################

#############################################################################
# MODULE DESCRIPTION
#
# Name: graph
# Language: python
#
# Purpose:
#  The purpose of this module is to generate a plot for a set of specified
#  input data.
#
# Function:
#  This program creates a window which will show a plot for the specified 
#  columns in a file. It allows to include conditions to select which files
#  to plot.
# 
# Dependencies:
#  None
#
# Files modified:
#  .plot.lock
#
# Files read:
#  None
#
# Resources usage:
#  See Design Document
#############################################################################

#############################################################################
#       RELEASE_HISTORY
# -------------
#          gLAB v1.3.0
# Release: 2009/12/10
# Change Log: First public version.
# -------------
#          gLAB v1.4.0
# Release: 2010/06/21
# Change Log: Save figure function added.
# -------------
#          gLAB v1.4.1
# Release: 2010/07/09
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.2
# Release: 2010/07/31
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.3
# Release: 2010/08/31
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.4
# Release: 2010/09/22
# Change Log: No changes in this file.
# -------------
#          gLAB v1.4.5
# Release: 2010/10/31
# Change Log: Minor Changes in help 
# -------------
#          gLAB v2.0.0
# Release: 2012/12/31
# Change Log: gLAB version released together with the Educational Book:
#             "GNSS Data Processing" (Vol. 1 and Vol. 2). ESA TM-23.
#             Authors: J. Sanz Subirana, J.M. Juan Zornoza and M. Hernandez-Pajares
#             ISBN: 978-92-9221-885-0 (two volumes)
#             ISSN: 1013-7076
#             December 2012.
# -------------
#          gLAB v2.2.0
# Release: 2014/09/22
# Change Log: No changes in this file.
# -------------
#          gLAB v2.2.1
# Release: 2014/12/23
# Change Log: No changes in this file.
# -------------
#          gLAB v2.2.2
# Release: 2015/03/02
# Change Log: No changes in this file.
# -------------
#          gLAB v2.2.3
# Release: 2015/05/28
# Change Log: No changes in this file.
# -------------
#          gLAB v2.2.4
# Release: 2015/07/01
# Change Log: No changes in this file.
# -------------
#          gLAB v2.2.5
# Release: 2015/10/20
# Change Log: When plot fails due to non-decimal values, the file .plot.lock is erased,
#               so gLAB_GUI.py "Plot" button doesn't get locked during 60 seconds. Also,
#               the error message has been clarified
#             In the Windows version (.exe binary), stdout has been redirected to stderr
#               so the help message is not lost
# -------------
#          gLAB v3.0.0
# Release: 2016/09/05
# Change Log: Added stanford plots and stanford-ESA plots
#             Added worst integrity ratio plots
#             Display environment in linux is not necessary when plot is saved to a file
#             It is now compatible with python 3
#             NOTE: The new plots (stanford and worst integrity ratio) need additional 
#                   python libraries (numpy and basemap), but they are not mandatory 
#                   if these plots are not used (for compatibility with previous versions)
# -------------
#          gLAB v3.1.0
# Release: 2016/09/26
# Change Log: Fixed a bug with input parameters due to bad space indendation 
# -----------
#          gLAB v3.1.1
# Release: 2016/10/21
# Change Log: Changed title name in Stanford-ESA plots from '# epochs' to '# samples'.
#             Fixed Stanford-ESA plots, which was horizontal protection levels from vertical plots and viceversa.
#             Added quantization check in Stanford plots generation for points greater than the
#                maximum X and Y. For example, (60,100) and (100,60) with a max X and Y of 50, 
#                would both result in the plot in the point (50,50) or (Xmax,Ymax), which made
#                it impossible to distinguish the MI from the non-MI. Now, MI points will result
#                in (Xmax,Ymax) while non MI will result in (Xmax-1,Ymax)
# -----------
#       END_RELEASE_HISTORY
#############################################################################

import sys,os,matplotlib,math,optparse,re
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from optparse import OptionParser
sys.stdout=sys.stderr


global legend
legend=False
class Plot:
    def __init__(self):
        self.FileName=""
        self.ColX="var[0]"
        self.ColY="var[1]"
        self.ColZ="" 
        self.Cond=""
        self.Label=""
        self.Xvar = []
        self.Yvar = []
        self.Zvar = []
        self.Style="."
        self.MarkerSize=5
        self.LineWidth=0
        self.PlotColor=""
        self.PlotColorAuto = True
        
        
class Graphic:
    def __init__(self):
        self.PlotList=[]
        self.PlotCounter = 0
        self.Title = ""
        self.Xlabel = ""
        self.Ylabel = ""
        self.FractionalTitle = ""

        self.Xmin = 0.0
        self.Xmax = 0.0
        self.Ymin = 0.0
        self.Ymax = 0.0
        
        self.XminAuto = True
        self.XmaxAuto = True
        self.YminAuto = True
        self.YmaxAuto = True
        
        self.SaveFigure = False
        self.SaveFigurePath = []
        
        self.grid = True
        
        self.stanford = False
        self.ALAuto = True 
        
        self.stanfordESA = False
        self.resolution_x_auto = True
        self.resolution_y_auto = True
        self.maxpoints_auto = True
        self.worstIntegrityRatio = False
        
        self.projection = "cyl"
        self.cbarAutoMin = True
        self.cbarAutoMax = True
        self.cbarAutoN = True

        self.continentColor = ""
        self.lakeColor = ""
        self.boundaryColor = ""

###########################################################################
# This part makes the Stanford Plot
# It is inspired by the code from http://kom.aau.dk/~borre/easy2/easy14/
#    Copyright c 1998 The board of trustees of the Leland Stanford     
#    Junior University. All rights reserved. 
#    This script file may be distributed and used freely, provided     
#     this copyright notice is always kept with it.                     
#                                                                       
#     Questions and comments should be directed to Todd Walter at:      
#     walter@relgyro.stanford.edu                                                   
###########################################################################
class PLstat:
    def __init__(self):
        self.AL = 35
        self.PL = []
        self.PE = []
        self.zMax = 0
        self.myPointPatch = []
        self.myPointColors = []
        self.n_fail1 = 0
        self.n_fail2 = 0
        self.n_fail3 = 0
        self.n_alarm = 0
        self.n_avail = 0
        self.n_total = 0
        self.Xmin = 0.0
        self.Xmax = 50.0
        self.Ymin = 0.0
        self.Ymax = 50.0
        self.resolution_x = 0.5
        self.resolution_y = 0.5
        self.Xlabel = 'Positioning Error (metres)'
        self.Ylabel = 'Protection Level (metres)'
        self.Title = 'Stanford Plot' 
        self.fig = plt.figure()
        self.ax = self.fig.add_subplot(111)
        self.fColor=['r','pink','m','y']
        self.grid = True
        self.sigma = False
        self.SaveFigure = False
        self.SaveFigurePath = ""
        self.cm = 'jet'

    def dealData(self):
        self.n_total=len(self.PL)
        newPE = []
        newPL = []
        self.intervalNo_x = 50.0/self.resolution_x
        self.intervalNo_y = 50.0/self.resolution_y
        self.data = np.zeros((int(self.intervalNo_x),int(self.intervalNo_y)))
        self.err_bin = np.linspace(self.resolution_x/2, (50-self.resolution_x/2), self.intervalNo_x)
        self.sig_bin = np.linspace(self.resolution_y/2, (50-self.resolution_y/2), self.intervalNo_y)
        for idx in range(self.n_total):
            j = self.PE[idx]
            k = self.PL[idx]
            diff = abs(k)-abs(j)
            j = int(abs(j)/self.resolution_x)
            k = int(abs(k)/self.resolution_y)
            if j >=(self.intervalNo_x-1):
                #When values are over maximum plot area, we need to check if they are MI.
                #This is because a value of (100,60) will have the same position in grid that (60,100),
                #making both values counted as MI epochs when there is only one.
                #There, values over maximum over plot area which are not MI will be set in position self.intervalNo_x-2
                if diff>0:
                    j = self.intervalNo_x-2
                else:
                    j = self.intervalNo_x-1
            newPE.append(j)
            if k >(self.intervalNo_y-1):
                k = self.intervalNo_y-1
            newPL.append(k)     
        for idx in range(self.n_total):
            self.data[int(newPE[idx]),int(newPL[idx])] = self.data[int(newPE[idx]),int(newPL[idx])] + 1
        
        return self
    
    def getPoints(self):
        self.zMax = math.ceil(math.log10(np.max(self.data)))
        aa,bb = np.nonzero(self.data)
        self.x = self.err_bin[aa]
        self.y = self.sig_bin[bb]
        
        delta_x = 0.5 * np.ones(len(aa)) * self.resolution_x
        delta_y = 0.5 * np.ones(len(bb)) * self.resolution_y
        x = self.x - delta_x
        y = self.y - delta_y
        wx = 2 * delta_x
        wy = 2 * delta_y
        mypatch = []
        mycolors = []
        for x1,y1,w,l in zip(x, y, wx, wy):
            rectangle = Rectangle((x1,y1), w, l, linewidth=0)
            mypatch.append(rectangle)
        for idx in range(len(aa)):
            z = math.log10(self.data[aa[idx],bb[idx]])
            c_idx = math.ceil(255*z/self.zMax)
            mycolors.append(c_idx)
        self.myPointPatch = mypatch
        self.myPointColors = np.array(mycolors)
        i_fails = np.where(self.x / self.y >= 1.0)
        i_fails = i_fails[0]
        for idx in i_fails:
            if self.x[idx] < self.AL:
                self.n_fail2 = self.n_fail2 + self.data[aa[idx],bb[idx]]
            elif self.y[idx] >= self.AL:
                self.n_fail3 = self.n_fail3 + self.data[aa[idx],bb[idx]]
            else:
                self.n_fail1 = self.n_fail1 + self.data[aa[idx],bb[idx]]
                
        i_alarm = np.where(self.y >= self.AL)
        i_alarm = i_alarm[0]
        self.n_alarm = sum(self.data[aa[i_alarm],bb[i_alarm]])
        self.n_avail = self.n_total - self.n_alarm - self.n_fail1 - self.n_fail2 
        return self
    
    def getPercentileCount(self):  
        text_style1 = dict(horizontalalignment='right', verticalalignment='top',
                  fontsize=12, fontdict={'family': 'monospace'})
        text_style2 = dict(horizontalalignment='center', verticalalignment='center',
                  fontsize=12, fontdict={'family': 'monospace'}, rotation=90)
        percentPoint = np.array([0.68, 0.95,0.999])
        percentIndex=percentPoint*len(self.PE)
        self.xPoint=np.zeros(3)
        self.yPoint=np.zeros(3)
        x=sorted(self.PE)
        y=sorted(self.PL)
        for i in range(3):
            self.xPoint[i]=round(x[int(round(percentIndex[i]))],2)
            self.yPoint[i]=round(y[int(round(percentIndex[i]))],2)

        self.ax.plot(self.xPoint,np.array([self.Ymin,self.Ymin,self.Ymin]),'^',markersize=10)
        self.ax.plot(np.array([self.Xmax,self.Xmax,self.Xmax]),self.yPoint,'<',markersize=10)
        for per, yp in zip(percentPoint,self.yPoint):
            self.ax.text((0.97*self.Xmax+0.03*self.Xmin),yp,'{percent:.1%}'.format(percent=per),**text_style1) 
        for per, xp in zip(percentPoint,self.xPoint):
            self.ax.text(xp,(0.06*self.Ymax+0.94*self.Ymin),'{percent:.1%}'.format(percent=per),**text_style2)     
        return self    
        
    def plotPoints(self):     
        p = PatchCollection(self.myPointPatch, cmap=plt.get_cmap(self.cm), linewidth=0)       
        p.set_array(self.myPointColors)
        self.ax.add_collection(p)
        m = self.zMax + 1
        cbarTicks = np.linspace(0, 255, m)
        cbarLabel = np.linspace(0, m-1, m)
        
        
        pCbar = plt.colorbar(mappable=p, ticks=cbarTicks)
        pCbar.ax.set_yticklabels(cbarLabel) 
        pCbar.ax.set_ylabel('Number of Points per Pixel (logarithmic)')
        self.ax.text(self.AL/5, 6*self.AL/7, 'Normal Operation \nEpochs: %d' %self.n_avail, style='italic')
        self.ax.text(self.AL/5, 5.5*self.AL/7, '{percent:.3%}'.format(percent=self.n_avail / float(self.n_total)), style='italic')
        return self
    
    def plotPatch(self):        
        
        fail1=Rectangle((self.AL, self.Ymin), (self.Xmax - self.AL), self.AL, lw=1, facecolor=self.fColor[0], alpha=0.2 )
        self.ax.add_patch(fail1)
        self.ax.text(8*self.AL/7, 1*self.AL/7, 'HMI \nEpochs: %d' %self.n_fail1 , style='italic')
        
        vert2 = [(self.Xmin, self.Ymin), (self.AL, self.AL),(self.AL, self.Ymin), (self.Xmin,self.Ymin),]
        code2 = [Path.MOVETO, Path.LINETO, Path.LINETO, Path.CLOSEPOLY,]
        path2 = Path(vert2, code2)
        fail2=PathPatch(path2, facecolor=self.fColor[1], lw=1, alpha=0.2)
        self.ax.add_patch(fail2)
        self.ax.text(2*self.AL/5, 1*self.AL/7, 'MI \nEpochs: %d' %self.n_fail2 , style='italic')
        
        vert3 = [(self.AL, self.AL), (self.Xmax, self.Ymax), (self.Xmax, self.AL), (self.AL, self.AL),]
        code3 = [Path.MOVETO, Path.LINETO, Path.LINETO, Path.CLOSEPOLY,]
        path3 = Path(vert3, code3)
        fail3=PathPatch(path3, facecolor=self.fColor[2], lw=1, alpha=0.2)
        self.ax.add_patch(fail3)
        self.ax.text(8*self.AL/7, 17*self.AL/16, 'MI \nEpochs: %d' %self.n_fail3 , style='italic')
        
        vert1 = [(self.Xmin, self.AL), (self.Xmin, self.Ymax), (self.Xmax, self.Ymax), (self.AL, self.AL), (self.Xmin, self.AL),]
        code1 = [Path.MOVETO, Path.LINETO, Path.LINETO, Path.LINETO, Path.CLOSEPOLY,]
        path1 = Path(vert1, code1)
        alarm=PathPatch(path1, facecolor=self.fColor[3], lw=1, alpha=0.2)
        self.ax.add_patch(alarm)
        self.ax.text(2*self.AL/5, 8*self.AL/7, 'System Unavailable \nAlarm Epochs: %d' %self.n_alarm , style='italic')
        return self
    
    def plotShow(self):
        plt.axis([self.Xmin, self.Xmax, self.Ymin, self.Ymax])
        self.ax.set_xlabel(self.Xlabel)
        self.ax.set_ylabel(self.Ylabel)
        self.ax.set_title(self.Title + ' (%d epochs)' %self.n_total )
        if self.sigma:
            ax2 = self.ax.twinx()
            ax2.set_yticks(self.yPoint)
            ax2.set_ylim(self.ax.get_ylim())
            ax2.set_yticklabels(self.yPoint)
            bb= np.linspace(self.Xmin,self.Xmax,5)
            cc=np.append(self.xPoint,bb)
            cc=sorted(cc)
            self.ax.set_xticks(cc)
            self.ax.set_xticklabels(cc,rotation='vertical')
        if self.grid:                 
            plt.grid(True)
        if self.SaveFigure:
            plt.savefig(self.SaveFigurePath)
        else:
            plt.show()
        
###########################################################################
# This part makes the StanfordESA Plot
###########################################################################
class stanfordESA:
    def __init__(self):
        self.zMax = 0
        self.f = ""
        self.myPointPatch = []
        self.myPointColors = []
        self.Xmin = 0.0
        self.Xmax = 60.0
        self.Ymin = 0.0
        self.Ymax = 60.0
        self.resolution_x = 0.1
        self.resolution_y = 0.1
        self.Xlabel = 'Positioning Error (metres)'
        self.Ylabel = 'Protection Level (metres)'
        self.Title = 'StanfordESA Plot' 
        self.grid = True
        self.sigma = False
        self.SaveFigure = False
        self.SaveFigurePath = ""
        self.cm = 'jet'
        self.maxpoints=500000
    def readData(self):
        f = open(self.f,'r')
        self.x = []
        self.y = []
        self.verticalNo = []
        self.horizontalNo = []
        i = 0
        for line in f:
            var=line.split()
            if i == 0:
                self.resolution_x = float(var[0])
                self.resolution_y = float(var[1])
                self.Xmax = float(var[2])
                self.Ymax = float(var[3])
                number_pixel=(self.Xmax*self.Ymax)/(self.resolution_x*self.resolution_y)
                i = i + 1
                if number_pixel> self.maxpoints:
                    print ("Number of pixels in plot (%d) is over limit (%d).\nReduce the pixel resolution, or reduce x&y range or increase the maximum number of pixels." %(number_pixel, self.maxpoints))
                    print ("To increase the maximum number of pixels, use the parameter '--maxpoints <value>'.\nWarning: If maximum number of pixels is too large, the system may run out of memory.")
                    sys.exit()
            else:
                self.x.append(float(var[0]))
                self.y.append(float(var[1]))
                self.horizontalNo.append(float(var[2]))
                self.verticalNo.append(float(var[3]))
        f.close()
        return self
        
    
    def getPoints(self,x,y,z):
        self.n_total = sum(z)
        self.n_fail = 0
        x = np.array(x)
        y = np.array(y)
        z = np.array(z)
        idx_nonzero = np.nonzero(z)
        x=x[idx_nonzero]
        y=y[idx_nonzero]
        z=z[idx_nonzero]
        self.zMax = math.ceil(math.log10(np.max(z)))

        mypatch = []
        mycolors = []
        for x1,y1 in zip(x, y):
            rectangle = Rectangle((x1,y1), self.resolution_x, self.resolution_y, linewidth=0)
            mypatch.append(rectangle)
        for number in z:
            c_idx = math.ceil(255*math.log10(number)/self.zMax)
            mycolors.append(c_idx)
        self.myPointPatch = mypatch
        self.myPointColors = np.array(mycolors)
        i_fails = np.where(x / y > 1.0)[0]
        for idx in i_fails:
            self.n_fail = self.n_fail + z[idx] 
        return self
       
    def plotPoints(self, ax):     

        p = PatchCollection(self.myPointPatch, cmap=plt.get_cmap(self.cm), linewidth=0)       
        p.set_array(self.myPointColors)
        ax.add_collection(p)
        m = self.zMax + 1
        cbarTicks = np.linspace(0, 255, m)
        cbarLabel = np.linspace(0, m-1, m)
        pCbar = plt.colorbar(mappable=p, ticks=cbarTicks)
        pCbar.ax.set_yticklabels(cbarLabel) 
        pCbar.ax.set_ylabel('Number of Points per Pixel (logarithmic)')
        x = [self.Xmin, self.Xmax]
        y = x
        plt.plot(x,y,'black')
        ax.text(2*self.Xmax/5, self.Ymax/5, 'N Geoms PE>PL: %d' %self.n_fail , style='italic')
        ax.text(17*self.Xmax/20, self.Ymax/20, 'gAGE/UPC', style='italic', fontsize = 8)
        plt.axis([self.Xmin, self.Xmax, self.Ymin, self.Ymax])
        ax.set_xlabel(self.Xlabel)
        ax.set_ylabel(self.Ylabel)
        ax.set_title(self.Title + ' (%d samples)' %self.n_total )
        if self.grid:                 
            plt.grid(True)
        return 
    
       


##########################################################################
# This part makes the worst integrity ratio plot
#
#
##########################################################################
class WIRmap:
    def __init__(self):
        self.fig =  plt.figure()
        self.gs = matplotlib.gridspec.GridSpec(2, 1, height_ratios=[20, 1]) 
        self.projection = 'cyl'
        self.Xmin = -180
        self.Xmax = 180
        self.Ymin = -90
        self.Ymax = 90
        self.continentColor = 'y'
        self.lakeColor = 'w'
        self.boundaryColor = 'w'
        #self.myColorMap = 'jet'
        self.longitude = []
        self.latitude = []
        self.ratio = []
        self.SaveFigure = False
        self.SaveFigurePath = ""
        self.Title = "Equidistant Cylindrical Projection"
        self.cbarMin = 0.0
        self.cbarMax = 2.0
        self.cbarInterval = 8.0
       
    def integrityRatioPlot(self):
        ms = int(np.sqrt((360.*180.)/(self.Xmax-self.Xmin)/(self.Ymax-self.Ymin))*4)
        ax0=self.fig.add_subplot(self.gs[0])
        if self.projection == 'cyl':
            self.m = Basemap(projection='cyl',llcrnrlat=self.Ymin,urcrnrlat=self.Ymax,\
                    llcrnrlon=self.Xmin,urcrnrlon=self.Xmax,resolution='l')
            self.parallel = np.arange(self.Ymin,self.Ymax+1.,10.)
            self.meridian = np.arange(self.Xmin,self.Xmax+1.,10.)  
        elif self.projection == 'lcc':
            self.m = Basemap(width=12000000,height=9000000,projection='lcc',\
                        resolution='l',lat_1=45.,lat_0=50,lon_0=10.)
            self.Title = "Lambert Conformal Projection"
            self.parallel = np.arange(0.,81.,10.)
            self.meridian = np.arange(10,351.,20.)  
        self.m.drawcoastlines()
        self.m.fillcontinents(color=self.continentColor,lake_color=self.lakeColor,alpha=0.5)
        self.m.drawparallels(self.parallel,labels=[1,0,0,0],fontsize=10)
        self.m.drawmeridians(self.meridian,labels=[0,0,0,1],fontsize=10)
        self.m.drawmapboundary(fill_color=self.boundaryColor)
        n = len(self.longitude)
        for i in range(n):
            c = (self.ratio[i]-self.cbarMin)/(self.cbarMax-self.cbarMin)
            if c < 0: c = 0
            if c > 1: c = 1
            xpt = self.longitude[i]
            ypt = self.latitude[i]
            r=str(round(self.ratio[i],1))
            c = matplotlib.cm.jet(c,1)
            if self.projection == 'cyl': 
                indomain = (xpt>=self.Xmin and xpt<=self.Xmax and ypt <= self.Ymax and ypt >= self.Ymin) 
                if indomain:
                    self.m.plot(xpt,ypt,'o', color = c, markersize=ms, alpha=0.8)
                    if ms >= 12:
                        plt.text(xpt, ypt, r, ha='center', va='center', fontsize=7, color='w')
            elif self.projection == 'lcc':
                xpt,ypt = self.m(xpt,ypt)
                self.m.plot(xpt,ypt,'o', color = c)

        plt.title(self.Title) 
       
        ax1=self.fig.add_subplot(self.gs[1])   
        cmap=plt.get_cmap('jet')
        bounds = np.linspace(self.cbarMin, self.cbarMax, self.cbarInterval+1)
        norm = matplotlib.colors.BoundaryNorm(bounds, cmap.N)
        matplotlib.colorbar.ColorbarBase(ax1, cmap=cmap, norm=norm, ticks=bounds, orientation='horizontal')
        if self.SaveFigure:
            plt.savefig(self.SaveFigurePath)
        else:
            plt.show()

##########################################################################
# Name        : Parse
# Description : This function reads from the command line all the 
#               options, and assigns them into the variables.
##########################################################################  
def Parse(value):
    auxValue=""
    mode=0
    try:
        number = str(int(value)-1)
        auxValue = "var["+number+"]"
    except:
        for element in value:
            if (mode==1):
                try:
                    number = str(int(element))
                    #auxValue=auxValue+number
                    n=n+number
                except:
                    mode=0
                    auxValue=auxValue+str(int(n)-1)+"]"
                    #auxValue=auxValue+"]"
            if (mode==0):
                if (element=="$"):
                    auxValue=auxValue+"var["
                    mode=1
                    n=""
                else:
                    auxValue=auxValue+element
        if (mode==1):
            auxValue=auxValue+str(int(n)-1)+"]"
    return auxValue
    
##########################################################################
# Name        : ParseCallback
# Description : When a new option for the graphic is introduced, this 
#               callback is called for storing the new parameter
##########################################################################   
def ParseCallback(option, opt_str, value, parser, Graph):

    if (option.dest=="Title"):
        #Graph.Title = value
        Graph.Title = unicode(value, "utf-8", errors="ignore")

    if (option.dest=="FractionalTitle"):
        #Graph.FractionalTitle = value
        Graph.FractionalTitle = unicode(value, "utf-8", errors="ignore")

    if (option.dest=="Xlabel"):
        #Graph.Xlabel = value
        Graph.Xlabel = unicode(value, "utf-8", errors="ignore")
    
    if (option.dest=="Ylabel"):
        #Graph.Ylabel = value
        Graph.Ylabel = unicode(value, "utf-8", errors="ignore")
    
    if (option.dest=="filename"):
        Graph.PlotCounter += 1
        Graph.PlotList.append(Plot())
        Graph.PlotList[Graph.PlotCounter-1].FileName = value
    
    if (option.dest=="ColX"):
        value = Parse(value)
        Graph.PlotList[Graph.PlotCounter-1].ColX = value
    
    if (option.dest=="ColY"):
        value = Parse(value)
        Graph.PlotList[Graph.PlotCounter-1].ColY = value

    if (option.dest=="ColZ"):
        value = Parse(value)
        Graph.PlotList[Graph.PlotCounter-1].ColZ = value

    if (option.dest=="Cond"):
        value = Parse(value)
        Graph.PlotList[Graph.PlotCounter-1].Cond = value

    if (option.dest=="PlotLabel"):
        global legend
        legend=True 
        #Graph.PlotList[Graph.PlotCounter-1].Label = value        
        Graph.PlotList[Graph.PlotCounter-1].Label = unicode(value, "utf-8", errors="ignore")        

    if (option.dest=="Style"):
        Graph.PlotList[Graph.PlotCounter-1].Style = value

        if (value.count('-')!= 0): 
            Graph.PlotList[Graph.PlotCounter-1].LineWidth = 1.0

    if (option.dest=="MarkerSize"):
        Graph.PlotList[Graph.PlotCounter-1].MarkerSize = float(value)

    if (option.dest=="LineWidth"):
        Graph.PlotList[Graph.PlotCounter-1].LineWidth = float(value)
        
    
    if (option.dest=="grid"):
        if (value == "off" or value == "no" ): Graph.grid = False   
        
    if (option.dest=="Xmin"):
        if (str(value) == '*' or value == "auto" ): Graph.XminAuto = True
        else:
            Graph.XminAuto = False
            Graph.Xmin = float(value)
                
    if (option.dest=="Xmax"):
        if (str(value) == '*' or value == "auto" ): Graph.XmaxAuto = True
        else:
            Graph.XmaxAuto = False
            Graph.Xmax = float(value)
        
    if (option.dest=="Ymin"):
        if (str(value) == '*' or value == "auto" ): Graph.YminAuto = True
        else:        
            Graph.YminAuto = False
            Graph.Ymin = float(value)
    
    if (option.dest=="Ymax"):
        if (str(value) == '*' or value == "auto" ): Graph.YmaxAuto = True
        else:
            Graph.YmaxAuto = False
            Graph.Ymax = float(value)

    if (option.dest=="PlotColor"):
        Graph.PlotList[Graph.PlotCounter-1].PlotColorAuto = False
        Graph.PlotList[Graph.PlotCounter-1].PlotColor = value
      
    if (option.dest=="SaveFigure"):
        Graph.SaveFigure = True 
        Graph.SaveFigurePath.append(str(value))
        
    if (option.dest=="AL"):
        if (str(value) == '*' or value == "auto" ): Graph.ALAuto = True
        else:
            Graph.ALAuto = False
            Graph.AL = float(value)


    if (option.dest=="resolution_x"):
        if (str(value) == '*' or value == "auto" ): Graph.intervalNo_x_auto = True
        else:
            Graph.intervalNo_x_auto = False
            Graph.intervalNo_x = int(value)
    if (option.dest=="resolution_y"):
        if (str(value) == '*' or value == "auto" ): Graph.intervalNo_y_auto = True
        else:
            Graph.intervalNo_y_auto = False
            Graph.intervalNo_y = int(value)
            
    if (option.dest=="maxpoints"):
        if (str(value) == '*' or value == "auto" ): Graph.maxpoints_auto = True
        else:
            Graph.maxpoints_auto = False
            Graph.maxpoints = int(value)
    
    
    if (option.dest=="projection"):
       if (value == "lcc" or value == 'lambert'): Graph.projection = "lcc"               
    
    if (option.dest=="cbarMin"):
        Graph.cbarAutoMin = False
        Graph.cbarMin = float(value)
    if (option.dest=="cbarMax"):
        Graph.cbarAutoMax = False
        Graph.cbarMax = float(value)
    if (option.dest=="cbarInterval"):
        Graph.cbarAutoN = False
        Graph.cbarInterval = float(value)
    if (option.dest=="continentColor"):
        Graph.continentColor = value
    if (option.dest=="lakeColor"):
        Graph.lakeColor = value
    if (option.dest=="boundaryColor"):
        Graph.boundaryColor = value



##########################################################################
# Name        : readFile
# Description : Read the input file, according to the plotting condition,  
#               assign the value to Xvar and Yvar
##########################################################################   
def readFile(fileN, cond, colX, colY, colZ):
    f = open(fileN,'r')
    Xvar = []
    Yvar = []
    Zvar = []
    if colZ != "":
        thirdInput = True
    for line in f:
        var=line.split()
        j=0
        for ele in var:
            try:
                auxEle = float(ele)
                var[j] = auxEle
                j += 1
            except:
                j += 1
        if (cond != ""):
            try:
                ret = eval(cond)
            except:
                ret = False
        else:
            ret = True
        if (ret==True):
            error = 0
            try:
                ret1 = eval(colX)
            except:
                error = 1
            
            try:
                ret2 = eval(colY)
            except:
                error = 1
            try:
                ret3 = eval(colZ)
            except:
                thirdInput = False

            if (error==0):
                Xvar.append(ret1)
                Yvar.append(ret2)
                
            if thirdInput:
                Zvar.append(ret3)
    f.close()
    return Xvar, Yvar, Zvar 
        
     
##########################################################################
# Name        : MakeGraph
# Description : This function makes the graphic, from the parameters
#               already parsed.
##########################################################################

        
def MakeGraph():
    i=0
    if Graph.stanford:
        sfp = PLstat()
        sfp.PE, sfp.PL, empty = readFile(Graph.PlotList[i].FileName, Graph.PlotList[i].Cond, Graph.PlotList[i].ColX, Graph.PlotList[i].ColY, Graph.PlotList[i].ColZ)
        if (Graph.ALAuto == False): sfp.AL = Graph.AL
        if (Graph.resolution_x_auto == False): sfp.resolution_x = Graph.resolution_x
        if (Graph.resolution_y_auto == False): sfp.resolution_y = Graph.resolution_y
        sfp.dealData()
        sfp.getPoints()
        sfp.plotPoints()
        if (Graph.XmaxAuto == False): sfp.Xmax = Graph.Xmax 
        if (Graph.XminAuto == False): sfp.Xmin = Graph.Xmin 
        if (Graph.YmaxAuto == False): sfp.Ymax = Graph.Ymax 
        if (Graph.YminAuto == False): sfp.Ymin = Graph.Ymin 
        if Graph.clean:
            x = [sfp.Xmin,sfp.Xmax]
            y = x
            plt.plot(x, y,'black')
        else:
            sfp.plotPatch()
        if (Graph.Xlabel != ""): sfp.Xlabel =Graph.Xlabel
        if (Graph.Ylabel != ""): sfp.Ylabel =Graph.Ylabel
        if (Graph.Title != ""): sfp.Title =Graph.Title
        if (Graph.sigma == True):
             sfp.sigma = Graph.sigma
             sfp.getPercentileCount()
        sfp.grid = Graph.grid
        sfp.SaveFigure = Graph.SaveFigure
        if (sfp.SaveFigure == True):
            sfp.SaveFigurePath = Graph.SaveFigurePath[0]
        sfp.plotShow()
    elif Graph.stanfordESA:
        sfESA = stanfordESA()
        sfESA.f = Graph.PlotList[i].FileName
        if (Graph.maxpoints_auto == False): sfESA.maxpoints = Graph.maxpoints
        sfESA.readData()
        if (Graph.resolution_x_auto == False): sfESA.resolution_x = Graph.resolution_x
        if (Graph.resolution_y_auto == False): sfESA.resolution_y = Graph.resolution_y
        if (Graph.XmaxAuto == False): sfESA.Xmax = Graph.Xmax 
        if (Graph.XminAuto == False): sfESA.Xmin = Graph.Xmin 
        if (Graph.YmaxAuto == False): sfESA.Ymax = Graph.Ymax 
        if (Graph.YminAuto == False): sfESA.Ymin = Graph.Ymin 
        if (Graph.Xlabel != ""): sfESA.Xlabel =Graph.Xlabel
        if (Graph.Ylabel != ""): sfESA.Ylabel =Graph.Ylabel
        if (Graph.Title != ""): sfESA.Title =Graph.Title
        sfESA.grid = Graph.grid
        sfESA.SaveFigure = Graph.SaveFigure
        if (sfESA.SaveFigure == True):
           sfESA.SaveFigurePath1 = Graph.SaveFigurePath[0]
        x = sfESA.x
        y = sfESA.y
        vertical = sfESA.verticalNo
        horizontal = sfESA.horizontalNo
        
        fig1 = plt.figure()
        ax1 = fig1.add_subplot(111)
        sfESA.getPoints(x,y,vertical)
        sfESA.Xlabel = "VPE"
        sfESA.Ylabel = "VPL"
        sfESA.plotPoints(ax1)
        if (sfESA.SaveFigure == True):
            plt.savefig(Graph.SaveFigurePath[0])
        fig2 = plt.figure()
        ax2 = fig2.add_subplot(111)
        sfESA.getPoints(x,y,horizontal)
        sfESA.Xlabel = "HPE"
        sfESA.Ylabel = "HPL"
        sfESA.plotPoints(ax2)

        if (sfESA.SaveFigure ==True) and (len(Graph.SaveFigurePath) > 1):
            plt.savefig(Graph.SaveFigurePath[1])
        if (sfESA.SaveFigure == False):
            plt.show()
    elif Graph.worstIntegrityRatio:
        from mpl_toolkits.basemap import Basemap
        wir=WIRmap()
        wir.longitude, wir.latitude, wir.ratio = readFile(Graph.PlotList[i].FileName, Graph.PlotList[i].Cond, Graph.PlotList[i].ColX, Graph.PlotList[i].ColY, Graph.PlotList[i].ColZ)
        wir.projection = Graph.projection
        if (Graph.XmaxAuto == False): wir.Xmax = Graph.Xmax 
        if (Graph.XminAuto == False): wir.Xmin = Graph.Xmin 
        if (Graph.YmaxAuto == False): wir.Ymax = Graph.Ymax 
        if (Graph.YminAuto == False): wir.Ymin = Graph.Ymin 
        if (Graph.Title != ""): wir.Title =Graph.Title
        if (Graph.cbarAutoMin == False): wir.cbarMin = Graph.cbarMin
        if (Graph.cbarAutoMax == False): wir.cbarMax = Graph.cbarMax
        if (Graph.cbarAutoN == False): wir.cbarInterval = Graph.cbarInterval
        if (Graph.continentColor != ""): wir.continentColor = Graph.continentColor
        if (Graph.lakeColor != ""): wir.lakeColor = Graph.lakeColor
        if (Graph.boundaryColor != ""): wir.boundaryColor = Graph.boundaryColor
        wir.SaveFigure = Graph.SaveFigure
        if (wir.SaveFigure == True):
            wir.SaveFigurePath = Graph.SaveFigurePath[0]
        wir.integrityRatioPlot()
        
    else:
        while i<Graph.PlotCounter:
            Graph.PlotList[i].Xvar, Graph.PlotList[i].Yvar, Graph.PlotList[i].Zvar = readFile(Graph.PlotList[i].FileName, Graph.PlotList[i].Cond, Graph.PlotList[i].ColX, Graph.PlotList[i].ColY, Graph.PlotList[i].ColZ)
            try:
                    if (Graph.PlotList[i].PlotColorAuto == False):
                        pylab.plot(Graph.PlotList[i].Xvar,Graph.PlotList[i].Yvar,Graph.PlotList[i].Style,label=Graph.PlotList[i].Label,color=Graph.PlotList[i].PlotColor,markersize=Graph.PlotList[i].MarkerSize,linewidth=Graph.PlotList[i].LineWidth)
                    else:
                        pylab.plot(Graph.PlotList[i].Xvar,Graph.PlotList[i].Yvar,Graph.PlotList[i].Style,label=Graph.PlotList[i].Label,markersize=Graph.PlotList[i].MarkerSize,linewidth=Graph.PlotList[i].LineWidth)
            except:
                    os.remove(".plot.lock")
                    print sys.exc_info()[1]
                    sys.exit()
    
            i+=1
        
        pylab.xlabel(Graph.Xlabel)
        pylab.ylabel(Graph.Ylabel)
        pylab.title(Graph.Title)
        if (Graph.FractionalTitle != ""):
            pylab.annotate(Graph.FractionalTitle, xy=(.025, .975), xycoords='figure fraction', horizontalalignment='left', verticalalignment='top')
    
        if (Graph.XmaxAuto == False): pylab.xlim( xmax=Graph.Xmax )
        if (Graph.XminAuto == False): pylab.xlim( xmin=Graph.Xmin )
        if (Graph.YmaxAuto == False): pylab.ylim( ymax=Graph.Ymax )
        if (Graph.YminAuto == False): pylab.ylim( ymin=Graph.Ymin )
        
        if Graph.grid:
            pylab.grid(True)
        
    
    #######        pylab.legend(numpoints=1, borderpad=0.25,handlelength=20,columnspacing=1,ncol=2)
    # ok    pylab.legend(numpoints=1, handlelength=0.2) 
    #        pylab.legend(numpoints=1,  borderpad=0.2)
    #DEF:
    #       pylab.legend(numpoints=1, borderpad=0.2,bbox_to_anchor=(0.85, 1), lo    c=2, ncol=1)
    #       pylab.legend(numpoints=1, borderpad=0.2,bbox_to_anchor=(-0.08, 0.975    ), loc=2, ncol=6, handlelength=-0.0, columnspacing=0.50)
    
        if (legend==True):
            pylab.legend()
            #pylab.legend(numpoints=1,bbox_to_anchor=(1.12, 1)) #Use this to align the label to the upper right corner (outside the plot)
            #pylab.legend(numpoints=1,bbox_to_anchor=(1, 1)) #Use this to align the label to the upper right corner (inside the plot)
            #pylab.legend(numpoints=1,bbox_to_anchor=(0.32, 1)) #Use this to align the label to the upper left corner (after the scale)
            #pylab.legend(numpoints=1,bbox_to_anchor=(0.32, .22)) #Use this to align the label to the bottom left corner (after the scale)
            #pylab.legend(numpoints=1,bbox_to_anchor=(.42,1)) 
            #pylab.legend(numpoints=1,bbox_to_anchor=(1, .22)) #Use this to align the label to the bottom right corner (inside the plot)
            #pylab.legend(numpoints=1,bbox_to_anchor=(1.12, .22)) #Use this to align the label to the bottom right corner (outside the plot)
            #pylab.legend(numpoints=1)  #Only one point in the legend
        
        if (os.path.isfile(".plot.lock")==True):
            os.remove(".plot.lock")
            
        if (Graph.SaveFigure==True):
            pylab.savefig(Graph.SaveFigurePath[0])
        else:   
            pylab.show()
        
        
        
        

        
##########################################################################
# Name        : 
# Description : Main execution thread
##########################################################################
if (os.path.isfile(".plot.lock") != True) :
    PlotLockFile = open(".plot.lock", "w")   
    PlotLockFile.close()
  
Graph = Graphic()


HelpHeader = """
#############################################################################
# Copyright: gAGE/UPC & ESA
# Project: EDUNAV GNSS Lab Tool
# Authors: Adria Rovira-Garcia (group of Astronomy and GEomatics - gAGE/UPC)
#          Pere Ramos-Bosch (group of Astronomy and GEomatics - gAGE/UPC)
# Curator: Adria Rovira-Garcia ( gAGE/UPC )
# Developer: Yixie Shao ( gAGE/UPC )
#            Deimos Ibanez Segura ( gAGE/UPC )
#          glab.gage @ upc.edu           
# File: graph.py
# Code Management Tool File Version: 3.1  Revision: 1
# Date: 2016/10/21
#############################################################################


GRAPHIC GENERAL OPTIONS:

  The General Options are the ones that can be specified once per graphic,
  and will affect the entire graphic window:
  
  -h, --help                    Show this help message and exit
  
  -t, --title, --tit            Set the title of the Graphic
  
  --Xlabel, --xlab, --xl        Set the x-axis label
  
  --Ylabel, --ylab, --yl        Set the y-axis label
  
  --Xmin, --xmin, --xn          The minimum value for the x axis to be plot, if 
                                no value is provided, automatic limits are set.  
                                
  --Xmax, --xmax, --xx          The maximum value for the x axis to be plot, if 
                                no value is provided, automatic limits are set.
                                
  --Ymin, --ymin, --yn          The minimum value for the y axis to be plot, if 
                                no value is provided, automatic limits are set.
                                
  --Ymax, --ymax, --yx          The maximum value for the y axis to be plot, if 
                                no value is provided, automatic limits are set. 

  --Ftitle, --ftitle, --ft      Add a fractional title to the upper left corner.
  
 
  Example:
       graph.py -t "Graphic title" --xl "time (s)" --yl "Altitude [m]"
      --xmin -3.0 --ymax 5.0


PLOT DEPENDENT OPTIONS:

  The Plot Dependent Options are specific to each plot.
  One new plot is considered from the point that a '-f' or '--file' is found. 
  All the options coming after this parameter belong to that specific plot.
  
  -f, --file                    Set the input file name for the specific plot.
  
  -x, -X, --Xcol                Set the source of the x axis: 
                                -x 4 or -x '$4' : will take as x axis the 4th
                                column of the input file.
                                
                                Operations can be done in this parameter, such as:
                                -x '($4-$5)': This will take the difference between
                                the 4th and the 5th column 
                                
                                Mathematical functions and constants are supported:
                                -x '(math.sin($12*math.pi/180)'
                                
  -y, -Y, --Ycol                Set the source of the y axis
                                Identical properties as x column

  -z, -Z, --Zcol                Set the source of the third input if it is necessary.
                                Identical properties as x column
                                
  -c, --cond                    Specify the plotting condition:
                                -c '($6>=10)' Include the 6th column if it is 
                                equal or greater than 10
                                
                                -c '($1==\"OUTPUT\")' Include the 1st column by a specific
                                string: surrounding the string by \"  
                                
                                Mathematical functions, booleans and constants are supported:
                                -c ($1==\"POSTFIT\")&(math.e($12+$5)>5.0)
                                
  -l, --label, --plotlabel      Sets the label for the current plot
  
  -s, --style                   Sets the style for the current plot  
                                The following styles are supported:
                                '.'     circle marker [DEFAULT]
                                '-'     solid line style
                                '--'    dashed line style
                                '-.'    dash-dot line style
                                'o'     point marker
                                's'     square marker
                                'p'     pentagon marker
                                '+'     plus marker
                                'x'     x marker
                                
  --color, --cl                 Sets the colour for the current plot
                                The following colours are supported:
                                'b'     blue [DEFAULT]
                                'g'     green
                                'r'     red
                                'c'     cyan
                                'm'     magenta
                                'y'     yellow
                                'k'     black
                                'w'     white

  --markersize, --ms            Sets the plot maker size 5 by [DEFAULT]

  --linewidth, --lw             Sets the plot maker size 0 by [DEFAULT]
  
  -g, --grid                    Set the plot with grid by [DEFAULT]
                                It could be turn off by 'off' or 'no'

  Example:
       graph.py -f /home/gLAB/Example -x '(math.sin($1*math.pi/180))' -y 2 
       -l "Plot Label" --style p --color cyan   
                                   

                                

MULTIPLE PLOTS:

  Each graphic window can contain several plots. One new plot is considered 
  from the point that a '-f' or '--file' is found:
      Graphic General Options: will apply to all the plots present
      Plot Dependent options: parameters specific to each plot.
  
  Example:
       graph.py -f /home/gLAB/Example -x '1' -y '2' -f /home/gLAB/Example -x '2' -y '1'


SAVE CURRENT GRAPHIC:

  User can save the entire graphic window into the path specified. If no path 
  is specified, it will be allocated in the current working directory.
  The graphic output format is deduced from the extension of the file name, if no
  extension is provided a 'png' extension type will be used: 
                         
  
  --save, --sv                  Stores the graphic window given a file name. The graphic
                                output format is deduced from the extension of the file name.
                                Supported formats are: 
                                emf, eps, jpeg, jpg, pdf, png [DEFAULT], ps, raw, rgba, svg, svgz.
                                
  Example:
       graph.py -f /home/gLAB/Example -x 1 -y 2 --save Figure_Example
       graph.py -f /home/gLAB/Example -x 1 -y 2 --sv /home/Desktop/Figure_Example.eps



       
STANFORD PLOT:

  In this mode: 
  
  For GRAPHIC GENERAL OPTIONS, all the options work the same way as in the other kind of the plot
                               except the option "--Ftitle", add a fractional title, is blocked. 
                               
  For PLOT DEPENDENT OPTIONS, the options "-f", "-x", "-y", "-c", "-g" work the same way as in the other kind of plot,
                              while the other options in this part are blocked in this mode.
                              "-x" and "-y" option can also be written as "--error" and "--pl" respectively
                              
  MULTIPLE PLOTS are disabled in this mode.
  
  For SAVE CURRENT GRAPHIC, it works the same way as for the other kind of plot.
  
  --stanford, --sf, --sp        Make a stanford plot.
                                
  --AL,       --al              Set the alarm limit for the protection level, if 
                                no value is provided, AL is set to 35 [DEFAULT].                                              
  --clean                       Make a stanford Plot without failure patches.
  --xr,   --xresolution         Set the resolution in x-direction of the plot. 
                                If no value is provided, it is set to 0.5 [DEFAULT].
  --yr,   --yresolution         Set the resolution in y-direction of the plot. 
                                If no value is provided, it is set to 0.5 [DEFAULT].
 Example:
        graph.py -f /home/gLAB/Example -x 1 -y 2 --sf
        

STANFORD-ESA PLOT:

  In this mode: 
  
  The input file should have the fixed format:
  
  The first row should provide the values: x_resolution y_resolution xmax ymax
  Since the second row, each row should provide the values: vertical_PE, vertical_PL, horizontal_PE, horizontal_PL

  For GRAPHIC GENERAL OPTIONS, all the options work the same way as in the other kind of the plot
                               except the option "--Ftitle", add a fractional title, is blocked. 
                               
  For PLOT DEPENDENT OPTIONS, the options "-f" works the same way as in the other kind of plot,
                              while the other options in this part are blocked in this mode.
                              
  MULTIPLE PLOTS are disabled in this mode.
  
  For SAVE CURRENT GRAPHIC, since it has two plots, then two save paths should be provided, if only one is provided,
                            only the vertical PL/PE will be saved.

  --stanfordESA, --sfesa, --spesa        Make a stanford-ESA plot.

 Example: 
       graph.py -f /home/gLAB/Example --sfesa --sv /home/Desktop/Figure_V.eps --sv /home/Desktop/Figure_H.eps
                                
 Example:
        graph.py -f /home/gLAB/Example --sfesa


WORST INTEGRITY RATIO PLOT:
  In this mode:

  The input file should contain the following fields: station geodetic coordinates (in degrees) and its worst integrity ratio.
  Parameter "-x": longitude (degrees)
  Parameter "-y": latitude (degrees)
  Parameter "-z": worst integrity ratio.

  For GRAPHIC GENERAL OPTIONS, all the options work the same way as in the other kind of the plot
                               except:
                                     the setting of "--xmax","--xmin","--ymax","--ymin" works 
                                     properly in the 'Equidistant Cylindrical Projection'.

                                     the option "--Ftitle", add a fractional title, is blocked. 
                                 
  For PLOT DEPENDENT OPTIONS, the options "-f", "-x", "-y", "-z" work the same way as in the other kind of plot,
                              while the other options in this part are blocked in this mode.
  
  MULTIPLE PLOTS are disabled in this mode.
  
  For SAVE CURRENT GRAPHIC, it works the same way as for the other kind of plot.
       
  --map                         Make a worst integrity ratio plot.

  --projection, --pj            Set the projection of the map. 
                                'Equidistant Cylindrical Projection' is set as [DEFAULT]
                                User can set the value of projection as 'lcc' or 'lambert' 
                                to switch to "Lambert Conformal Projection"

  --cbarMin,--cbarmin,--cmin    The minimum value for the colorbar, if no value is provided,
                                automatic limits are set.  

  --cbarMax,--cbarmax,--cmax    The maximum value for the colorbar, if no value is provided,
                                automatic limits are set. 

  --cbarInterval,--cbarN,--cn   The value of interval for colorbar's tick, if no value is provided,
                                8 is set as [DEFAULT]

  --continentColor,--cc         The continent's color, if no value is provided, 'yellow' is 
                                set as [DEFAULT]

  --lakeColor, --lc             The lake's color, if no value is provided, 'white' is 
                                set as [DEFAULT]

  --boundaryColor,--bc          The continent's color, if no value is provided, 'white' is 
                                set as [DEFAULT]
 Example:
        graph.py -f /home/gLAB/Example --map -x 3 -y 4 -z 5 
 
"""

GraphicParser = OptionParser(usage=HelpHeader) 

GraphicParser.add_option("--title","--tit","-t", type="string", dest="Title", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ftitle","--ftitle","--ft", type="string", dest="FractionalTitle", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xlabel","--xlab","--xl" ,type="string", dest="Xlabel", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ylabel","--ylab","--yl" ,type="string", dest="Ylabel", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--file","-f" ,type="string", dest="filename", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xcol","-x","-X", "--error",type="string", dest="ColX", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ycol","-y","-Y", "--pl", type="string",dest="ColY", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Zcol","-z","-Z", type="string",dest="ColZ", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--cond","-c", type="string",dest="Cond", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--label","--plotlabel","-l", type="string",dest="PlotLabel", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--style","-s", type="string",dest="Style", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP) 

GraphicParser.add_option("--ms","--MarkerSize","--markersize",type="string",dest="MarkerSize", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP) 
GraphicParser.add_option("--lw","--LineWidth","--linewidth", type="string",dest="LineWidth", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP) 

GraphicParser.add_option("--save","--sv", type="string",dest="SaveFigure", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--color","--cl", type="string",dest="PlotColor", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xmin","--xmin","--xn", type="string",dest="Xmin", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Xmax","--xmax","--xx", type="string",dest="Xmax", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ymin","--ymin","--yn", type="string",dest="Ymin", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--Ymax","--ymax","--yx", type="string",dest="Ymax", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)

GraphicParser.add_option("--grid","-g", type="string",dest="grid", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)


GraphicParser.add_option("--stanford","--sf","--sp", dest="stanford", default=False,action='store_true',help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--clean",dest="clean", default=False,action='store_true',help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--AL","--al",type="string",dest="AL", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--xr","--xresolution",type="string",dest="resolution_x", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--yr","--yresolution",type="string",dest="resolution_y", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--sigma", dest="sigma", default=False,action='store_true',help=optparse.SUPPRESS_HELP)

GraphicParser.add_option("--stanfordESA","--sfesa","--spesa", dest="stanfordESA", default=False,action='store_true',help=optparse.SUPPRESS_HELP)

GraphicParser.add_option("--maxpoints",type="string",dest="maxpoints", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)

GraphicParser.add_option("--map",dest="wir", default=False,action='store_true',help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--projection","--pj", type="string",dest="projection", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--cbarMin","--cbarmin","--cmin", type="string",dest="cbarMin", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--cbarMax","--cbarmax","--cmax", type="string",dest="cbarMax", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--cbarInterval","--cbarN","--cn", type="string",dest="cbarInterval", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--continentColor","--cc", type="string",dest="continentColor", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--lakeColor","--lc", type="string",dest="lakeColor", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)
GraphicParser.add_option("--boundaryColor","--bc", type="string",dest="boundaryColor", action="callback",callback=ParseCallback,callback_args=(Graph,),help=optparse.SUPPRESS_HELP)


(opts,args)=GraphicParser.parse_args()

if(opts.stanford and opts.wir):
    print "ERROR: stanford plots and worst integrity ratio plots cannot be both set"
    sys.exit()

if(opts.stanford and opts.stanfordESA):
    print "ERROR: stanford plots and stanford-ESA plots cannot be both set"
    sys.exit()


if(opts.wir and opts.stanfordESA):
    print "ERROR: stanford-ESA plots and worst integrity ratio plots cannot be both set"
    sys.exit()

if(opts.stanford and opts.stanfordESA and opts.wir):
    print "ERROR: stanford plots, stanford-ESA plots and worst integrity ratio plots cannot be all set at once"
    sys.exit()


if(opts.stanford or opts.wir or opts.stanfordESA):
    if (Graph.PlotCounter>1):
        print "ERROR: stanford plots or worst integrity ratio plots or stanford-ESA plots only allow one plot per execution"
        sys.exit()

if (opts.stanford):
    Graph.stanford = True 
    Graph.clean = False
    Graph.sigma = False
if (opts.stanfordESA):
    Graph.stanfordESA = True 
   

if (opts.wir):
    Graph.worstIntegrityRatio = True
    
if (opts.clean):
    Graph.clean = True
if (opts.sigma):
    Graph.sigma = True
#matplot lib Agg does not use XDisplay, so if we are saving plot in a file, we will not need XDisplay in Linux
if Graph.SaveFigure:
    import matplotlib
    matplotlib.use('Agg')

#The imports are done after the parameters are read, so if no stanford or 
#worst integrity ratio plots are done, their libraries are not imported
if (Graph.worstIntegrityRatio or Graph.stanford or Graph.stanfordESA):
    import numpy as np
    import matplotlib.pyplot as plt
    from matplotlib.path import Path
    from matplotlib.patches import Rectangle,PathPatch
    from matplotlib.collections import PatchCollection
else:
    import pylab
if Graph.worstIntegrityRatio:
    from mpl_toolkits.basemap import Basemap

MakeGraph()
